@@ -42,12 +42,11 @@ extern "C" {
 
 /* API from R */
 
-SEXP processx_exec(SEXP command, SEXP args,
-		   SEXP std_in,  SEXP std_out, SEXP std_err, SEXP pty,
-		   SEXP pty_options, SEXP connections, SEXP env,
-		   SEXP windows_verbatim_args,
-		   SEXP windows_hide_window, SEXP private_, SEXP cleanup,
-		   SEXP wd, SEXP encoding, SEXP tree_id);
+SEXP processx_exec(SEXP command, SEXP args, SEXP pty, SEXP pty_options,
+		   SEXP connections, SEXP env, SEXP windows_verbatim_args,
+		   SEXP windows_hide_window, SEXP windows_detached_process,
+		   SEXP private_, SEXP cleanup, SEXP wd, SEXP encoding,
+		   SEXP tree_id);
 SEXP processx_wait(SEXP status, SEXP timeout, SEXP name);
 SEXP processx_is_alive(SEXP status, SEXP name);
 SEXP processx_get_exit_status(SEXP status, SEXP name);
@@ -61,14 +60,14 @@ SEXP processx_poll(SEXP statuses, SEXP conn, SEXP ms);
 
 SEXP processx__process_exists(SEXP pid);
 SEXP processx__proc_start_time(SEXP status);
-SEXP processx__unload_cleanup();
+SEXP processx__unload_cleanup(void);
 
 SEXP processx_is_named_pipe_open(SEXP pipe_ext);
 SEXP processx_close_named_pipe(SEXP pipe_ext);
 SEXP processx_create_named_pipe(SEXP name, SEXP mode);
 SEXP processx_write_named_pipe(SEXP pipe_ext, SEXP text);
 
-SEXP processx_disable_crash_dialog();
+SEXP processx_disable_crash_dialog(void);
 
 SEXP processx_base64_encode(SEXP array);
 SEXP processx_base64_decode(SEXP array);
@@ -78,6 +77,8 @@ SEXP processx_base64_decode(SEXP array);
 /* Interruption interval in ms */
 #define PROCESSX_INTERRUPT_INTERVAL 200
 
+/* These correspond to io.R, update there as well! */
+
 /* Various OSes and OS versions return various poll codes when the
    child's end of the pipe is closed, so we cannot provide a more
    elaborate API. See e.g. http://www.greenend.org.uk/rjk/tech/poll.html
@@ -96,11 +97,12 @@ SEXP processx_base64_decode(SEXP array);
 #define PXSILENT  5		/* still open, but no data or EOF for now. No timeout, either */
                                 /* but there were events on other fds */
 #define PXEVENT   6             /* some event, this is used for curl fds */
+#define PXCONNECT 7             /* a connection is available for a server socket */
 
 /* These statuses can be only returned by the pre-poll functions */
 
-#define PXHANDLE  7             /* need to poll the set handle */
-#define PXSELECT  8             /* need to poll/select the set fd */
+#define PXHANDLE  8             /* need to poll the set handle */
+#define PXSELECT  9             /* need to poll/select the set fd */
 
 typedef struct {
   int windows_verbatim_args;
