@@ -1,11 +1,6 @@
 
-## To resolve....
-online <- curl::has_internet()
-if (online) httpbin()
-
 test_that("curl fds", {
   skip_on_cran()
-  if (!online) skip("Offline")
 
   resp <- list()
   errm <- character()
@@ -13,53 +8,42 @@ test_that("curl fds", {
   fail <- function(x) errm <<- c(errm, x)
 
   pool <- curl::new_pool()
-  url1 <- httpbin("/status/200")
-  url2 <- httpbin("/delay/1")
-  curl::multi_add(pool = pool, curl::new_handle(url = url1),
+  url1 <- httpbin$url("/status/200")
+  url2 <- httpbin$url("/delay/1")
+  curl::multi_add(pool = pool, curl::new_handle(url = url1, http_version = 2),
                   done = done, fail = fail)
-  curl::multi_add(pool = pool, curl::new_handle(url = url1),
+  curl::multi_add(pool = pool, curl::new_handle(url = url1, http_version = 2),
                   done = done, fail = fail)
-  curl::multi_add(pool = pool, curl::new_handle(url = url2),
+  curl::multi_add(pool = pool, curl::new_handle(url = url2, http_version = 2),
                   done = done, fail = fail)
-  curl::multi_add(pool = pool, curl::new_handle(url = url1),
+  curl::multi_add(pool = pool, curl::new_handle(url = url1, http_version = 2),
                   done = done, fail = fail)
-  curl::multi_add(pool = pool, curl::new_handle(url = url1),
+  curl::multi_add(pool = pool, curl::new_handle(url = url1, http_version = 2),
                   done = done, fail = fail)
 
+  # This does not do much, but at least it tests that we can poll()
+  # libcurl's file descriptors
+
   timeout <- Sys.time() + 5
   repeat {
-    state <- curl::multi_run(timeout = 1/10000, pool = pool, poll = TRUE)
     fds <- curl::multi_fdset(pool = pool)
-    if (length(fds$reads) > 0) break;
-    if (Sys.time() >= timeout) break;
+    if (length(fds$reads) > 0) {
+      pr <- poll(list(curl_fds(fds)), 1000)
+    }
+    state <- curl::multi_run(timeout = 0.1, pool = pool, poll = TRUE)
+    if (state$pending == 0 || Sys.time() >= timeout) break;
   }
 
   expect_true(Sys.time() < timeout)
-
-  xfds <- list()
-  xpr <- character()
-
-  while (state$pending > 0) {
-    fds <- curl::multi_fdset(pool = pool)
-    xfds <- c(xfds, fds["reads"])
-    pr <- poll(list(curl_fds(fds)), 2000)
-    xpr <- c(xpr, pr[[1]])
-    state <- curl::multi_run(timeout = 0.1, pool = pool, poll = TRUE)
-  }
-
-  expect_true(all(vapply(xfds, length, 1L) > 0))
-  expect_true(all(xpr == "event"))
-
   expect_equal(vapply(resp, "[[", "", "url"), c(rep(url1, 4), url2))
 })
 
 test_that("curl fds before others", {
   skip_on_cran()
-  if (!online) skip("Offline")
 
   pool <- curl::new_pool()
-  url <- httpbin("/delay/1")
-  curl::multi_add(pool = pool, curl::new_handle(url = url))
+  url <- httpbin$url("/delay/1")
+  curl::multi_add(pool = pool, curl::new_handle(url = url, http_version = 2))
 
   timeout <- Sys.time() + 5
   repeat {
@@ -87,11 +71,10 @@ test_that("curl fds before others", {
 
 test_that("process fd before curl fd", {
   skip_on_cran()
-  if (!online) skip("Offline")
 
   pool <- curl::new_pool()
-  url <- httpbin("/delay/1")
-  curl::multi_add(pool = pool, curl::new_handle(url = url))
+  url <- httpbin$url("/delay/1")
+  curl::multi_add(pool = pool, curl::new_handle(url = url, http_version = 2))
 
   timeout <- Sys.time() + 5
   repeat {
